@@ -266,6 +266,9 @@ void scriptingInit(int setup) {
     luaSetTableProtectionRecursively(lua);
     lua_pop(lua, 1);
 
+    /* Set metatables of basic types (string, number, nil etc.) readonly. */
+    luaSetTableProtectionForBasicTypes(lua);
+
     lctx.lua = lua;
 }
 
@@ -273,6 +276,7 @@ void scriptingInit(int setup) {
 void freeLuaScriptsSync(dict *lua_scripts, list *lua_scripts_lru_list, lua_State *lua) {
     dictRelease(lua_scripts);
     listRelease(lua_scripts_lru_list);
+    lua_gc(lctx.lua, LUA_GCCOLLECT, 0);
     lua_close(lua);
 
 #if !defined(USE_LIBC)
